Package: trtf
Title: Transformation Trees and Forests
Version: 0.4-3
Date: 2025-01-29
Authors@R: person("Torsten", "Hothorn", role = c("aut", "cre"),
                  email = "Torsten.Hothorn@R-project.org", comment = c(ORCID = "0000-0001-8301-0471"))
Description: Recursive partytioning of transformation models with
  corresponding random forest for conditional transformation models 
  as described in 'Transformation Forests' (Hothorn and Zeileis, 2021, <doi:10.1080/10618600.2021.1872581>) 
  and 'Top-Down Transformation Choice' (Hothorn, 2018, <DOI:10.1177/1471082X17748081>).
Depends: mlt (>= 1.4-1), partykit (>= 1.2-1), tram
Imports: Formula, sandwich, grid, stats, variables, libcoin, utils,
        grDevices
Suggests: survival, TH.data, coin, ATR, Matrix, colorspace, inum,
        lattice, latticeExtra, memisc, multcomp, quantregForest,
        AppliedPredictiveModeling, mlbench
URL: http://ctm.R-forge.R-project.org
License: GPL-2
NeedsCompilation: no
Packaged: 2025-01-29 15:01:58 UTC; hothorn
Author: Torsten Hothorn [aut, cre] (<https://orcid.org/0000-0001-8301-0471>)
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2025-01-29 15:30:05 UTC
Built: R 4.6.0; ; 2026-04-28 12:37:38 UTC; unix
