################################################################################

# Model-based Random Forests for Ordinal Regression

# Muriel Buri and Torsten Hothorn

################################################################################

This folder contains simplified and commented R-scripts used for the empirical
evaluation and applied example of the manuscript "Model-based Random Forests for
Ordinal Regression".

Please refer to the manuscript for details on the functions, models and simulation
procedures.

In each subfolder you will find an instruction on how to run the experiments.

- The subfolder "empeval" is for comparison of prognostic models on the
  simulated data.

- The subfolder "ALS" is for the re-analysis of the "Respiratory" sub-item in
  functioncal rating scales of patients suffering from Amyotrophic Lateral
  Sclerosis (ALS).

