This is the folder with R-code implementation of the experiments described in
Korepanova N., Seibold H., Steffen V., Hothorn T. Survival Forests under Test:
Impact of the Proportional Hazards Assumption on Prognostic and Predictive 
Forests for ALS Survival (2019). 

In each subfolder you will find an instruction on how to run the experiments.
The subfolder "prognostic" is for comparison of prognostic models on the
simulated data.
The subfolder "predictive" is for comparison of predictive models on the
simulated data.
The subfolder "ALS" is for experiments in the samples generated from
ALS dataset.

The code in "ALS" subfolder source the code from "prognostic" and
"predictive" subfolders. Therefore, we recommend not to change the
structures of the current folder and its subfolders.
